package de.fhw.swtvertiefung.knauf.jsp3;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Wolfgang Knauf
 * 
 * Servlet fr das Verarbeiten der Requests der JSP-Seite. Wertet den Request
 * aus und errechnet das Ergebnis. Dadurch ist alles fr das Rendern der Ausgabe
 * bereit.
 *  
 */
public class GeometricModelServlet extends HttpServlet
{
  /**
   * Get-Requests verarbeiten.
   * 
   * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse)
   */
  protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
  {
    try
    {
      this.getServletContext().log(
          "Verarbeite Request mit a='" + request.getParameter("a") + "', b='" + request.getParameter("b") + "', c='"
              + request.getParameter("c") + "'");
      //Status der Session ausgeben:
      this.getServletContext().log(
          "Session.isNew: " + request.getSession().isNew() + ", Session-ID: " + request.getSession().getId());

      Seitenlaengen seitenlaengenAktuell = new Seitenlaengen();
      //Die Request-Parameter a, b und c auswerten und in die Seitenlngen
      // schreiben.
      double dblA, dblB, dblC;

      dblA = Double.parseDouble(request.getParameter("a"));
      dblB = Double.parseDouble(request.getParameter("b"));
      dblC = Double.parseDouble(request.getParameter("c"));

      seitenlaengenAktuell.setA(dblA);
      seitenlaengenAktuell.setB(dblB);
      seitenlaengenAktuell.setC(dblC);

      //Berechnen:
      double dblVolumen = dblA * dblB * dblC;
      double dblOberflaeche = 2 * (dblA * dblB) + 2 * (dblA * dblC) + 2 * (dblB * dblC);

      //In den Request schreiben, damit die JSP auswerten kann.
      request.setAttribute("Volumen", new Double(dblVolumen));
      request.setAttribute("Oberflaeche", new Double(dblOberflaeche));

      //Historie-Objekt aus der Session holen, falls vorhanden.
      Historie historie = null;
      //"getSession" erzeugt eine Session, falls nicht vorhanden.
      if (request.getSession().getAttribute("historie") != null)
      {
        this.log("Attribut 'historie' in Session gefunden");
        historie = (Historie) request.getSession().getAttribute("historie");
      }
      else
      {
        this.log("Attribut 'historie' NICHT in Session gefunden");
        historie = new Historie();
        //In Session schreiben:
        request.getSession().setAttribute("historie", historie);
      }
      historie.addSeitenlaenge(seitenlaengenAktuell);

      //Weiterleiten des Requests (Achtung: wir befinden uns hier in der
      //Seite "/servlet/GeometricModelServlet", d.h. eine Ebene hher wechseln !
      request.getRequestDispatcher("index.jsp").forward(request, response);
    }
    catch (Exception ex)
    {
      this.getServletContext().log("Fehler in doGet (" + ex.getClass().getName() + "): " + ex.getMessage());

      //Die Exception in den Context schreiben.
      //Unter diesem Namen sucht die Errorpage sie spter (siehe
      // JSP-Spezifikation oder Doku zu Tomcat5-Klasse
      // "org.apache.jasper.runtime.JspRuntimeLibrary").
      //Setzen dieses Attributs reicht damit JBoss einen Fehler erkennt
      //und auf die Errorpage weiterleitet.
      request.setAttribute("javax.servlet.error.exception", ex);
      //Wir knnten hier zustzlich einen Response-Fehlercode setzen...
      //response.sendError(500);

      request.getRequestDispatcher("errorpage.jsp").forward(request, response);
    }
  }

  /**
   * Post-Requests verarbeiten. Leitet direkt weiter auf doGet.
   * 
   * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse)
   */
  protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
  {
    this.doGet(request, response);
  }
}
